package  ddj;

import com.sun.media.jsdt.NameInUseException;

/**
 * The ChatException class is part of the design which
 * isolates the GUI layer from the middle layers and the
 * JSDT layers (See the comment on ChatApplicationGUI).
 * The middle layer throws only this exception, not
 * JSDTExceptions, to the GUI layer. The GUI layer
 * can get details about the exception, if needed, by
 * examining the Exception contained in the
 * ChatException.
 *
 * If you want your top layer to know about some of the
 * distinctions indicated by the different JSDTExceptions
 * but you still want to avoid coupling the layer to
 * the JSDT classes, you can make these distinction
 * by providing your own "enum"-like exception-type
 * variable. This is demonstrated here with the
 * client-name-in-use exception type.
 *
 * In fact, the JSDTExceptions also have an exception type
 * marked with an integer "enum"-like field (retrievable
 * with getType()). This duplicates the class of the
 * exception as a marker of exception type.
 */
public class ChatException extends Exception {
    public final static int UNSPECIFIED_EXCEPTION_TYPE = 0;
    public final static int CLIENT_NAME_IN_USE = 1;

    private Exception containedException;
    private int exceptionType = UNSPECIFIED_EXCEPTION_TYPE;

    /**
     *
     */
    public ChatException() {
        super();

    }

    /**
     *
     */
    public ChatException(String s) {
        super(s);
    }

    /**
     *
     */
    public ChatException(Exception e) {
        this();
        setContainedException(e);
    }

    /**
     *
     */
    public ChatException(String s, Exception e) {
        this(s);
        setContainedException(e);
    }

    /**
     *
     */
    private void  setContainedException(Exception e) {
        containedException = e;
        if (e instanceof NameInUseException) {
            exceptionType = CLIENT_NAME_IN_USE;
        }
    }

    /**
     *
     */
    public Exception getContainedException() {
        return containedException;
    }

    /**
     *
     */
    public int getExceptionType() {
        return exceptionType;
    }

    /**
     * One of the problems posed by rethrowing exceptions
     * is that stack-trace information is lost. This class
     * makes up for this by printing the stack trace of the
     * contained exception along with the stack trace of
     * this exceptions.
     */
    public void printStackTrace() {
        if (getContainedException() != null){
            getContainedException().printStackTrace();
        }
        super.printStackTrace();
    }

     /**
      * Print information about the contained Exception
      * along with the information about this exception.
      */
    public String toString() {
        String s = super.toString();
        Exception ex = getContainedException();
        if (ex != null) {
            s += "; Contained Exception: "  + ex.toString();
        }

        return s;
    }

}
