// Copyright (C) 1999 Joshua Fox
package ddj;

import com.sun.media.jsdt.Client;
import com.sun.media.jsdt.Session;
import com.sun.media.jsdt.JSDTException;
import com.sun.media.jsdt.NoSuchSessionException;
import com.sun.media.jsdt.event.SessionAdaptor;
import com.sun.media.jsdt.event.SessionEvent;


/**
 * The ChatSessionListener listens for the Session Events.
 * It lets the ChatApplicationGUI know when a client
 * has arrived or left.
 *
 * Notes:
 *  1. ChatSessionListener extends SessionAdaptor, which provides empty
 *      implementations of SessionListener.
 *  2. The SessionListener, ChannelListener, ByteArrayListener,
 *      TokenListener, and ClientListener do <I>not</I> listen to
 *      the Data that may flow through the Session, Channel, etc.
 *      Rather, they listen to <I>Events</I> concerning
 *      these objects, such as Clients joining the object,
 *      or the object being destroyed.
 */
public class ChatSessionListener extends SessionAdaptor {
    private ChatApplicationGUI chatGui;
    private ChatApplication chatApp;

    /**
     *
     */
    ChatSessionListener(ChatApplicationGUI  gui, ChatApplication app) {
        chatApp = app;
        chatGui = gui;
    }


   /**
   *
   */
  public void sessionJoined(SessionEvent sessionEvent) {
    try {
        // session should equal ChatApplication.currentSession;
        // You could put an assertion here as an error-check.
        Session session = sessionEvent.getSession();
        String [] names = session.listClientNames();
        chatGui.setClientsInSession(names);
    } catch (JSDTException jsdte){
        System.err.println("Exception " + jsdte + " in ChatSessionListener.sessionJoined()");
    }

  }

  /**
   *
   */
  public void sessionLeft(SessionEvent sessionEvent) {

     try {
        // session should equal ChatApplication.currentSession;
        // You could put an assertion here as an error-check.
        Session session = sessionEvent.getSession();

        String [] names = session.listClientNames();
        chatGui.setClientsInSession(names);
     } catch (NoSuchSessionException nsse) {
        // When we call disconnect() just to "clean up",
        // we will get this exception, so there is no need
        // to print anything.
     } catch (JSDTException jsdte){
        System.err.println("Exception " + jsdte + " in ChatSessionListener.sessionLeft()");
     }
  }


   /**
   *
   */
  public void sessionDestroyed(SessionEvent sessionEvent) {
        chatGui.setEndSession();
  }
}

