// Copyright (C) 1999 Joshua Fox 
package ddj;

import com.sun.media.jsdt.Client;
import com.sun.media.jsdt.AuthenticationInfo;

/**
 * The Client object represents the applicaton
 * (whether client-side or server-side)
 * in the JSDT system, by providing a name in
 * getName(), and by responding to authentication
 * challenges in authenticate().
 */
public class JSDTClient implements Client {
    private String clientName;

    /**
     * Constructor. Note that the most important feature
     * of the Client is its name, which identifies it
     * in the JSDT system.
     */
    public JSDTClient(String name) {
        clientName = name;
    }


    /**
     * Identifies this participant in the JSDT system.
     */
    public String getName() {
        return clientName;
    }


    /**
     * If this Client tries to enter a Managed object, it will
     * be met with a challenge (a Java Object). It will
     * give its response (another Java Object).
     * In a system with no authentication, like this one, both
     * challenge and response are null. If you want a secure
     * challenge, you will have to supply your own algorithm.
     */
    public Object authenticate(AuthenticationInfo info) {
    	return null;
    }

    /**
     * It's often a good idea to override String to get a meaningful
     * value for debugging purposes.
     */
    public String toString() {
        return getName();
    }
}
